@@ -426,6 +426,15 @@ void DeclarationBuilder::declareClassMember(DUContext *parentCtx, AbstractType::
                                                 AstNode* node )
 {
     if ( m_upcomingClassVariables.contains(identifier) ) {
+        if (m_actuallyRecompiling) {
+            DUChainWriteLocker lock;
+            Declaration* dec = currentContext()->parentContext()->findDeclarationAt(startPos(node));
+            if ( dynamic_cast<ClassMemberDeclaration*>(dec) ) {
+                // invalidate declaration, it got added
+                // see also bug https://bugs.kde.org/show_bug.cgi?id=241750
+                delete dec;
+            }
+        }
         return;
     }
 
@@ -660,7 +669,7 @@ void DeclarationBuilder::declareVariable(DUContext* parentCtx, AbstractType::Ptr
     DUChainWriteLocker lock(DUChain::lock());
     // check if this variable is already declared
     {
-        QList< Declaration* > decs = parentCtx->findDeclarations(identifier.first(), startPos(node));
+        QList< Declaration* > decs = parentCtx->findDeclarations(identifier.first(), startPos(node), 0, DUContext::DontSearchInParent);
         if ( !decs.isEmpty() ) {
             QList< Declaration* >::const_iterator it = decs.constEnd() - 1;
             while ( true ) {
